#ifndef	CIRCBUF_CLS
#define	CIRCBUF_CLS

#include	"userdefs.h"
#include	<_defs.h>
#include	<stddef.h>

#include	<stdio.h>

/*
The CircularBuffer class is a character buffer ring. It has the following
operations defined.

CircularBuffer(size=0)
~CircularBuffer()
int		Size(size=0)
int 	ReSize(size)
void	Flush()
char	Remove()
BOOL	Full()
BOOL	Empty()
char	*ErrorMessage()
int		ErrorCode()

A buffer may be created with a size or with default size zero. If size=0,
the buffer must then be initialised with the Size() function before it can
be used.

Size() returns the current size of the buffer.
Uninitialised buffer,
	Size(n)	returns n. For n>0, initialises the buffer to size n.
	If the buffer memory space could not be allocated, returns 0 and sets
	error code to 1.
Initialised buffer,
	Size(n) Returns the current size of the buffer, regardless of value of n.

ReSize() returns the current size of the buffer or 0.
If a non-empty buffer was the target of ReSize, Error=5.
If an attempt was made to ReSize an empty buffer to 0, Error=6.
If an attempt was made to ReSize an empty buffer to a size greater than 0,
	but the memory could not be allocated, returns 0 and sets Error=1.

Flush()
The pointers are set to initial values for an empty but initialised buffer.
Flush() should be called before ReSize() if the current contents of the
buffer are not required for any reason.

Add(c)
If there is room, the character c is added to the buffer.
If the buffer was already full Error=2.
If the buffer was not initialised Error=4.
If the buffer was empty, c is added and the empty flag is set to FALSE
If the buffer becomes full, the full flag is set to TRUE

Remove()
Returns a character from the buffer.
If the buffer was empty, returns NULL and sets Error=3
If the buffer was not initialised, returns NULL and sets Error=4
If the buffer was full, sets the full flag to FALSE
If the buffer becomes empty, sets the empty flag to TRUE

Full()
Returns TRUE if Full.

Empty()
Returns TRUE if empty.

ErrorCode()
Returns the current error code value.

ErrorMessage()
Returns an error message associated with the current error code value.

*/

_CLASSDEF(CircularBuffer)

class _CLASSTYPE	CircularBuffer {
private:
	char		_FAR *Buffer;
	BOOL		_Full,_Empty;
	int			Start,Finish;
	int			_Size;
	int			Error;
	static	const	char	*messages[];
	BOOL		Initialised;
public:
	CircularBuffer(const int size=0);
	~CircularBuffer() {	delete [] Buffer;	}
	const	int	_FAR		Size(const int _FAR n=0);
	const	int	_FAR		ReSize(const int _FAR n);
	void	_FAR				Flush();
	void	_FAR				Add(const char _FAR);
	char	_FAR				Remove();
	BOOL	_FAR				Full(){ return _Full;};
	BOOL	_FAR				Empty(){ return _Empty; };
	const	char	_FAR	*ErrorMessage();
	const	int		_FAR	ErrorCode(){ return Error; };
};

inline CircularBuffer::CircularBuffer(const int size):_Size(size),
											_Full(FALSE),_Empty(TRUE),Start(0),Finish(0),Error(0)
{
	if(_Size==0){ Initialised=FALSE; }
	else
	{
		if(!(Buffer=new char[size]))
		{
			Error=1;
			Initialised=FALSE;
		}
		else	Initialised=TRUE;
	}
}

inline	char _FAR CircularBuffer::Remove()
{
	char c=NULL;

	if(_Empty)
	{
		Error=3;
		return c;
	}
	if(!Initialised)
	{
		Error=4;
		return c;
	}
	else
	{
		c=Buffer[Finish];
		Finish=(Finish+1)%_Size;
		if (Finish==Start) _Empty=TRUE;
		_Full=FALSE;
		Error=0;
	}
	return c;
}

inline	const char _FAR *CircularBuffer::ErrorMessage()
{
	return	messages[Error];
}
inline const int _FAR CircularBuffer::Size(const int _FAR n)
{
	if(n && !Initialised)
	{
		if(!(Buffer=new char[n]))
		{
			Error=1;
			_Size=0;
		}
		else
		{
			Initialised=TRUE;
			_Size=n;
		}
	}
	return _Size;
}

inline	void	_FAR	CircularBuffer::Flush()
{
	Start=Finish=0;
	_Full=FALSE;
	_Empty=TRUE;
	Error=0;
}

#endif
